/**
 * @file
 * Attaches administration-specific behavior for the Filter module.
 */

(function ($, Drupal) {
  /**
   * Displays and updates the status of filters on the admin page.
   *
   * @type {Drupal~behavior}
   *
   * @prop {Drupal~behaviorAttach} attach
   *   Attaches behaviors to the filter admin view.
   */
  Drupal.behaviors.PageLinks = {
    attach(context, settings) {

      let editor = null;

      window.addEventListener("scroll", (event) => {
        let y = Math.abs(parseInt(document.body.getBoundingClientRect().top)) || 0;
        if (y < 150){
          $('#edit-page-links').removeClass('page-links-sticked');
        }
      });

      function getCkeditor() {
        if (!editor && $('.ck-content').length) {
          editor = Drupal.CKEditor5Instances.get(
            Drupal.CKEditor5Instances.keys().next().value,
          );
        }
      }

      setTimeout(function () {
        getCkeditor();
      }, 500);

      // Generate ftp dsn string by link click.
      $(once('deleteLink', '.delete-link')).on(
        'click',
        (event) => {
          event.preventDefault();

          let t = $(event.target);
          const Links = settings['PageLinks'] || [];
          console.log(Links, 'links');
          if (editor && $('.ck-content').length) {
              let link_id = t.attr('data-link-id');
              let link_order = t.attr('data-link-order');
              let clicked_link_id = link_order + '-' + link_id;
              let link_full = Links[clicked_link_id][0];
              let link_text = Links[clicked_link_id][2];
              console.log(clicked_link_id, 'clicked_link_id');
              let curValue = editor.getData();
              // Remove link by inner text.
              curValue = replaceLink(curValue,link_full, link_text, link_order);
              getCkeditor();
              // Delele link from editor.
              editor.setData(curValue);

              // Finally hide delete link.
              t.hide();
              // Mark roe as delted.
              let tr = t.parents('tr')
              $(tr).addClass('deleted');
              $(tr).find('td.url a').addClass('deleted');

              // Display Apply button
             $('#edit-page-links-wrapper').show();
             // Set removed links number.
             let button = $('#edit-page-links-wrapper .button');
             let removed_count = $(button).attr('data-removed-count') || 0;
             removed_count++;
             $(button).attr('data-removed-count', removed_count);
             $('.removed-messsage').html('Removed <b>'+ removed_count + '</b> links.');
          }

        return false;
        });

      // Generate ftp dsn string by link click.
      $(once('gotoLink', '.goto-link')).on(
        'click',
        (event) => {
          event.preventDefault();

          let t = $(event.target);
          // Do not scroll to removed link.
          if ($(t).hasClass('deleted')){
           return false;
          }

          const Links = settings['PageLinks'] || [];
console.log(Links, 'LINKS');
          if (editor && $('.ck-content').length) {
            let link_id = t.attr('data-link-id');
            let link_order = t.attr('data-link-order');
            let clicked_link_id = link_order + '-' + link_id;
            let link_full = Links[clicked_link_id][0];
            console.log(link_order + '-' + link_id, 'find link');

            getCkeditor();
            // Scroll editor area to clicked link.
            scrollToLink(editor, link_full, link_order);
            // stick the links block.
            $('#edit-page-links').removeClass('page-links-sticked').addClass('page-links-sticked');
          }

          return false;
        });

      function replaceLink(text, word, replacement, link_order) {
        let parts = text.split(word); // Split the text by the word
        let occurrenceCount = parts.length - 1; // Calculate the number of occurrences

        console.log(parts);
        console.log(occurrenceCount);
        // Only proceed if there are at least two occurrences
        if (occurrenceCount >= link_order) {
          // Rebuild the string with the first occurrence, the replacement, and the rest of the string

          let newText = parts.slice(0, link_order).join(word) + replacement + parts.slice(link_order).join(word);

          return newText;
        }

        // Return the original text if less than two occurrences
        return text;
      }


      function scrollToLink(editor, link_full, link_order) {
            const editorContent = $(editor.editing.view.domConverter.viewToDom(editor.editing.view.document.getRoot(), document));
            const editor_links = editorContent.find('a');
            console.log(editor_links, 'a editor_links');
            let editor_html = editor.getData();
            let links_ordered = getLinksOrdered(editor_html, editor_links);
            console.log(links_ordered, 'links_ordered');

            const textNode = editor_links.filter(
            function (i, el) {
              // Find the start position of the link's HTML in the entire body's HTML
              var start_pos = editor_html.indexOf(link_full);

             // console.log(getHash(el.outerHTML) , 'hash editor.link_full');
             // console.log(getHash(link_full), 'hash link_full');
             // console.log(links_ordered[i] + ' = ' + link_order, 'cmp link order');

              return getHash(el.outerHTML) === getHash(link_full) && links_ordered[i] == link_order;
            }).get(0);

        console.log(textNode, 'founded A');
            if (textNode) {
              $(textNode).get(0).scrollIntoView({ behavior: 'smooth', block: 'center' });
              // HighLight link.
              highLightLink(textNode);
            }
      }

      function getLinksOrdered(editorContent, links){
        console.log('getLinksOrdered');
        let map = {};
        let result = {};
        $.each(links, function(k, link){
        let link_full = link.outerHTML;
        let link_id = getHash(link_full);
          if (typeof map[link_id] !== 'undefined'){
            map[link_id] = map[link_id] +1;
          }
          else {
            map[link_id] = 1;
          }
          let link_order = map[link_id];

          let start_pos = editorContent.indexOf(link_full);
          result[k] = link_order;
        });

        console.log(map, 'map');
        console.log(result, 'result');
        return result;
      }


      function getHash(s) {
          /* Simple hash function. */
          var a = 1, c = 0, h, o;
          if (s) {
            a = 0;
            /*jshint plusplus:false bitwise:false*/
            for (h = s.length - 1; h >= 0; h--) {
              o = s.charCodeAt(h);
              a = (a<<6&268435455) + o + (o<<14);
              c = a & 266338304;
              a = c!==0?a^c>>21:a;
            }
          }
       //   console.log(s, String(a));
          return String(a);
        };


      /**
       * Highlight the scrolled link.
       */
      function highLightLink(link){
        $(link).css('background-color', 'yellow');
        setTimeout(() => { $(link).removeAttr('style'); }, 2000);
      }

    },
  };
})(jQuery, Drupal);
