<?php

namespace Drupal\page_links;

use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Render\Markup;
use Drupal\Core\Link;

/**
 * @todo Add class description.
 */
class PageLinkService {

  use StringTranslationTrait;

  const LINKS_PATTERN = '/<a[ ].*?href=["\']([^"\']+)["\'].*?>(.+?)<\/a>/i';

  public function themeLinksTable($links) {
    $header = ['Del', $this->t('Type'), $this->t('Link URL')];
    $rows = [];
    foreach ($links as $link_data) {
      $rows[] = [
        'data' => [
          [
            'data' => $this->prepareLink($link_data, 'delete'),
            'class' => 'del',
          ],
          ['data' => $this->getLinkType($link_data[1]), 'class' => 'type'],
          ['data' => $this->prepareLink($link_data), 'class' => 'url'],
        ],
        'class' => 'row-link-' . $this->getLinkType($link_data[1]),
      ];
    }

    $build = [
      '#type' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#attributes' => ['class' => ['page-links-table']],
    ];

    return $build;
  }

  /**
   * Return link type local (L) or remote (R).
   *
   * @param string $url
   *
   *
   * @return string
   */
  private function getLinkType($url) {
    $info = parse_url($url);
    return empty($info['scheme']) ? 'L' : 'R';
  }

  /**
   * Grab links array from html text.
   *
   * @param $text
   *
   * @return array|void
   */
  public function getLinks($text = '') {
    if (!$text) {
      return;
    }

    $pattern = self::LINKS_PATTERN;
    // Array to hold the matches
    $matches = [];
    // Execute the regex
    if (!preg_match_all($pattern, $text, $matches)) {
      return;
    }
    // Check and print the matches
    $rows = [];
    $count_values = [];
    foreach ($matches[1] as $key => $link) {
      $link_id = $this->hash($matches[0][$key]);

      if(isset($count_values[$link_id])){
         $count_values[$link_id]++;
       } else {
         $count_values[$link_id] = 1;
      }

      $link_order = $count_values[$link_id];

      $rows[$link_order . '-' . $link_id] = [
        $matches[0][$key],
        $matches[1][$key],
        $matches[2][$key],
        $link_id,
        $link_order
      ];
    }
    return $rows;
  }

  public function hash($s) {
    /* Simple hash function. */
    $a = 1;
    $c = 0;
    $h = 0;
    $o = 0;
    if ($s) {
      $a = 0;
      // Loop through the string backwards
      for ($h = strlen($s) - 1; $h >= 0; $h--) {
        $o = ord($s[$h]); // Get ASCII value of character
        $a = (($a << 6) & 268435455) + $o + ($o << 14);
        $c = $a & 266338304;
        $a = ($c !== 0) ? ($a ^ ($c >> 21)) : $a;
      }
    }
    return strval($a);
  }

  /**
   *  Get link by type.
   *
   * @param $link
   * @param $type
   *
   * @return \Drupal\Component\Render\MarkupInterface|string
   */
  public function prepareLink($link, $type = '') {
    $link_text = htmlentities($link[2]);
    $link_url = htmlentities($link[1]);
    $link_id = $link[3];
    $link_order = $link[4];

    if ($type == 'delete') {
      $result = '<a href="#" title="' . $this->t('Delete link') . '" data-link-order="' . $link_order . '" data-link-id="' . $link_id . '" class="delete-link action-link--danger">[x]</a>';
    }
    else {
      $result = '<a href="#" class="link goto-link" title="' . $link_text . '" data-link-id="' . $link_id . '" data-link-order="' . $link_order . '" >' . $link_url . '</a>';
    }

    return Markup::create($result);
  }

}
